//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                       (c) 1997 European Space Agency
//
//---------------------------------------------------------------------------
//
//	ColumnSelector.cc:	--- implementation of ColumnSelector class
//
//	CHANGE HISTORY:
//		UL, 1998-10-22: 1.0.0: - first version
//		UL, 1999-02-23: 2.0.0: - several changes; middle pane is now
//								 scrollable if the number of columns becomes
//								 too large; still a few rough edges ...
//		UL, 1999-03-01: 2.1.0: - check buttons associated with image creation
//								 now get a label "X"|"Y" indicating the
//								 type of axis
//		UL, 1999-09-21: 2.1.1: - re-aligned column-button column after Qt2 port
//		UL, 1999-10-28: 2.2.0: - bring up warning dialog if min>max
//		UL, 2000-02-11: 2.3.0: - changed layout of table column related
//								 widget; neater appearence
//							   - value types into spinbox widget does no
//								 longer default to 99 when the corresponding
//								 column has no TLMIN/TLMAX set
//		UL, 2000-05-10: 2.3.1: - made spacing between adjacent rows smaller
//		UL, 2000-07-18: 2.4.0: - various changes related to SPR 1678
//		UL, 2000-10-16: 2.5.0: - should not delete widgets in destructor
//								 unless they are orphans
//		UL, 2000-12-22: 2.5.1: - trivial change to make optimizer happy
//
//	AUTHOR:
//		U. Lammers (UL), Thu Oct 22 14:24:39 MET DST 1998
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/xmmselect/src/ColumnSelector.cc,v 1.22 2002/02/06 16:30:12 ulammers Exp $
//

#include <cstdlib>
#include <climits>

#include <qlayout.h>
#include <qtooltip.h>
#include <qframe.h>
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qscrollview.h>

#include "ColumnSelector.h"
#include "ExprConstructor.h"
#include "XmmSelectServer.h"
#include "stringutil.h"
#include "common.h"
#include "OnDemandScrollView.h"

//
//	constructor
//
ColumnSelector::ColumnSelector(QWidget *parent, const char *name,
			       ExpressionConstructor *exprConstructor,
			       const vector<TableColumn> &tableColumns)
	: QWidget(parent, name), _exprConstructor(exprConstructor),
	  _tableColumns(tableColumns),
	  _nextIndexToChange(0), _oneDprojectionCol(-1)
{
	_twoDprojectionCol[0] = _twoDprojectionCol[1] = -1;

	_layout = new QVBoxLayout(this, 0);

	//	create two button groups
	_twoDprojectionButtons = new QButtonGroup(this);
	_oneDprojectionButtons = new QButtonGroup(this);

	//	make the button groups invisible
	_twoDprojectionButtons->setGeometry(0, 0, 0, 0);
	_oneDprojectionButtons->setGeometry(0, 0, 0, 0);

	for (unsigned n=0; n<tableColumns.size(); ++n) {
		SelectionGroup *widget = addColumn(this, tableColumns[n]);
		widget->layout()->setResizeMode(QLayout::Minimum);

		_layout->addWidget(widget, 0, AlignLeft);
		registerColumn(widget, n);
	}

	connect(_twoDprojectionButtons, SIGNAL(pressed(int)),
			this, SLOT(twoDprojectionAxisSelected(int)));
	connect(_oneDprojectionButtons, SIGNAL(pressed(int)),
			this, SLOT(oneDprojectionAxisSelected(int)));

	this->setMinimumHeight(this->sizeHint().height());
} // ColumnSelector::ColumnSelector()


void ColumnSelector::addColumn(const TableColumn &tableColumn)
{
	SelectionGroup *widget = addColumn(this, tableColumn);
	_layout->addWidget(widget, 0);
	registerColumn(widget, _tableColumns.size()-1);
	widget->show();
	this->update();
} // ColumnSelector::addColumn()


SelectionGroup *ColumnSelector::addColumn(QWidget *parent,
										  const TableColumn &tableColumn)
{
	SelectionGroup *widget = 0;
	switch (tableColumn.type()) {
		case Dal::Int8:
		case Dal::Uint16:
		case Dal::Int16:
		case Dal::Uint32:
		case Dal::Int32:
			widget = new IntegerSelectionGroup(parent, tableColumn);
			connect(widget,
				SIGNAL(selected(const string &, int, int, int, int,
					   QPushButton *)),
				this,
				SLOT(integerSelection(const string &, int, int, int, int,
					 QPushButton *)));
			break;
		case Dal::Real32:
		case Dal::Real64:
			widget = new RealSelectionGroup(parent, tableColumn);
			connect(widget,
					SIGNAL(selected(const string &, double, double,
									bool, bool, QPushButton *)),
					this,
					SLOT(realSelection(const string &, double, double,
									   bool, bool, QPushButton *)));
			break;
		case Dal::DString:
			widget = new StringSelectionGroup(parent, tableColumn);
			connect(widget,
					SIGNAL(selected(const string &, const string &, bool)),
					this,
					SLOT(stringSelection(const string &, const string &,
						 bool)));
			break;
		case Dal::Bool:
			widget = new BooleanSelectionGroup(parent, tableColumn);
			connect(widget,
					SIGNAL(selected(const string &, bool, bool)),
					this,
						SLOT(booleanSelection(const string &, bool, bool)));
			break;
		default: {
			string s = "Column `" + tableColumn.name() +
					   "\' is of unknown type";
			QMessageBox::warning(xmmSelectServer->gui(),
				"Unknown column type", s.c_str(), "Ok");
			break;
		}
	}

	return widget;
} // ColumnSelector::addColumn()


void ColumnSelector::registerColumn(SelectionGroup *widget, int id)
{
	if (widget->twoDprojectionCheck())
		_twoDprojectionButtons->insert(widget->twoDprojectionCheck(), id);
	if (widget->oneDprojectionCheck())
		_oneDprojectionButtons->insert(widget->oneDprojectionCheck(), id);
} // ColumnSelector::registerColumn()


ColumnSelector::~ColumnSelector()
{
} // ColumnSelector::ColumnSelector()


const TableColumn *ColumnSelector::twoDprojectionAxisX() const
{
	if (_twoDprojectionCol[0] >= 0)
		return &_tableColumns[_twoDprojectionCol[0]];

	return 0;
} // ColumnSelector::twoDprojectionAxisX()


double ColumnSelector::twoDprojectionAxisXmin() const
{
	if (_twoDprojectionCol[0] >= 0) {
		const TableColumn &tc = _tableColumns[_twoDprojectionCol[0]];
		return tc.dmin();
	}		

	return 1.;
} // ColumnSelector::twoDprojectionAxisXmin()


const TableColumn *ColumnSelector::twoDprojectionAxisY() const
{
	if (_twoDprojectionCol[1] >= 0)
		return &_tableColumns[_twoDprojectionCol[1]];

	return 0;
} // ColumnSelector::twoDprojectionAxisY()


double ColumnSelector::twoDprojectionAxisYmin() const
{
	if (_twoDprojectionCol[1] >= 0) {
		const TableColumn &tc = _tableColumns[_twoDprojectionCol[1]];
		return DBL(tc.dmin());
	}		

	return 1.;
} // ColumnSelector::twoDprojectionAxisYmin()


const TableColumn *ColumnSelector::oneDprojectionAxis() const
{
	if (_oneDprojectionCol >= 0)
		return &_tableColumns[_oneDprojectionCol];

	return 0;
} // ColumnSelector::oneDprojectionAxis()


double ColumnSelector::oneDprojectionAxisMin() const
{
	if (_oneDprojectionCol >= 0) {
		const TableColumn &tc = _tableColumns[_oneDprojectionCol];
		return DBL(tc.dmin());
	}

	return 1.;
} // ColumnSelector::oneDprojectionAxisMin()


void ColumnSelector::booleanSelection(const string &colName, bool value, bool)
{
	string s;
	if (value)
		s = colName + "==true";
	else
		s = colName + "==false";

	_exprConstructor->appendExpression(s,
						   ExpressionConstructor::OR_IF_SAME_SENDER, colName);
} // ColumnSelector::booleanSelection()


void ColumnSelector::stringSelection(const string &colName,
				     const string &value, bool)
{
	string s = colName + "==\"" + value + '"';
	_exprConstructor->appendExpression(s,
						   ExpressionConstructor::OR_IF_SAME_SENDER, colName);
} // ColumnSelector::stringSelection()


void ColumnSelector::integerSelection(const string &colName,
									  int min, int max,
									  int colMinVal, int colMaxVal,
									  QPushButton *p)
				      
{
	string s;
	if (min>colMinVal && max<colMaxVal)
		if (min != max)
			if (min < max)
				s = colName + " in [" + toString(min) + ":" +
														toString(max) + "]";
			else {
				minGreaterThanMax(min, max, p);
				return;
			}
		else
			s = colName + "==" + toString(min);
	else if (min > colMinVal)
		s = colName + (min!=max ? ">=" : "==") + toString(min);
	else if (max < colMaxVal)
		s = colName + (min!=max ? "<=" : "==") + toString(max);

	if (!s.empty())
		_exprConstructor->appendExpression(s,
						   ExpressionConstructor::OR_IF_SAME_SENDER, colName);
	else {
		QMessageBox::information(xmmSelectServer->gui(),
								 "Expression with no effect",
								 "Both the lower and upper boundary is\n"
								 "identical to the absolute minimum/maximum\n"
								 "data value in that column - filtering "
								 "would\nhave no effect", "Ok");
		p->setDown(false);
	}
} // ColumnSelector::integerSelection()


void ColumnSelector::realSelection(const string &colName,
								   double min, double max,
								   bool minChanged, bool maxChanged,
								   QPushButton *p)
{
	string s;
	if (minChanged && maxChanged)
		if (min != max)
			if (min < max)
				s = colName + " in [" + toString(min, 12) + ":" +
													toString(max, 12) + "]";
			else {
				minGreaterThanMax(min, max, p);
				return;
			}
		else
			s = colName + "==" + toString(min, 12);
	else if (minChanged)
		s = colName + ">=" + toString(min, 12);
	else if (maxChanged)
		s = colName + "<=" + toString(max, 12);

	if (minChanged || maxChanged)
		_exprConstructor->appendExpression(s,
						   ExpressionConstructor::OR_IF_SAME_SENDER, colName);
	else {
		QMessageBox::information(xmmSelectServer->gui(),
								 "Expression with no effect",
								 "Neither lower nor upper boundary have been\n"
								 "given - filtering would have no effect.",
								 "Ok");
		p->setDown(false);
	}
} // ColumnSelector::realSelection()


void ColumnSelector::twoDprojectionAxisSelected(int id)
{
	QButton *button = _twoDprojectionButtons->find(id);
	if (_twoDprojectionCol[0] != id && _twoDprojectionCol[1] != id) {
		//
		//	new selection
		//
		if (_twoDprojectionCol[_nextIndexToChange] > 0) {
			//	unselect
			_twoDprojectionButtons->
				find(_twoDprojectionCol[_nextIndexToChange])->setDown(false);
			_twoDprojectionButtons->
				find(_twoDprojectionCol[_nextIndexToChange])->toggle();
			_twoDprojectionButtons->
				find(_twoDprojectionCol[_nextIndexToChange])->setText("");
		}
		_twoDprojectionCol[_nextIndexToChange] = id;
		button->setText(_nextIndexToChange ? "Y" : "X");
		_nextIndexToChange = (_nextIndexToChange + 1)%2;
	} else if (_twoDprojectionCol[0] == id) {
		_twoDprojectionCol[0] = -1;
		button->setText("");
		_nextIndexToChange = 0;
	} else if (_twoDprojectionCol[1] == id) {
		_twoDprojectionCol[1] = -1;
		button->setText("");
		_nextIndexToChange = 1;
	}
} // ColumnSelector::twoDprojectionAxisSelected()



void ColumnSelector::oneDprojectionAxisSelected(int id)
{
	_oneDprojectionCol = id;
} // ColumnSelector::oneDprojectionAxisSelected()


SelectionGroup::SelectionGroup(QWidget *parent, const string &colName)
	: QFrame(parent), _colName(colName),
	_twoDprojectionCheck(0), _oneDprojectionCheck(0)
{
} // SelectionGroup::SelectionGroup()


QGridLayout *SelectionGroup::setupCheckButtons(bool twoDprojCheck,
											   bool oneDprojCheck)
{
	QGridLayout *grid = new QGridLayout(1, 2, 2);
	grid->addRowSpacing(0, 10);
	for (unsigned n=0; n<2; grid->addColSpacing(n++, 20));

	if (twoDprojCheck) {
		_twoDprojectionCheck = new QPushButton(this);
		_twoDprojectionCheck->setToggleButton(true);
		_twoDprojectionCheck->setFixedWidth(12);
		_twoDprojectionCheck->setFixedHeight(12);
		QToolTip::add(_twoDprojectionCheck,
					  "Use this column in image accumulation");
		grid->addWidget(_twoDprojectionCheck, 0, 0, AlignVCenter);
	}

	if (oneDprojCheck) {
		_oneDprojectionCheck = new QRadioButton(this);
		QToolTip::add(_oneDprojectionCheck,
			  "Use this column in spectrum/histogram/ratescurve accumulation");
		grid->addWidget(_oneDprojectionCheck, 0, 1, AlignVCenter);
	}

	return grid;
} // SelectionGroup::setupCheckButtons()


SelectionGroup::~SelectionGroup()
{
} // SelectionGroup::~SelectionGroup()



IntegerSelectionGroup::IntegerSelectionGroup(QWidget *parent,
		  const TableColumn &tableColumn)
	: SelectionGroup(parent, tableColumn.name())
{
	int xWidth = this->fontMetrics().width('X');

	_grid = new QGridLayout(this, 1, 7, 10);

	//	integer column can be used in image and histogram/spectrum extraction
	_grid->addLayout(setupCheckButtons(true, true), 1, 1);

	_columnButton = new QPushButton(this);
	_columnButton->setText(name().c_str());
 	_columnButton->setMinimumSize(_columnButton->sizeHint());
 	_columnButton->setMinimumWidth(12 * xWidth);
	string s = "Append " + name() + " interval filter to selection expression";
	QToolTip::add(_columnButton, s.c_str());

	connect(_columnButton, SIGNAL(pressed()), this, SLOT(signalLimits()));
	this->setMinimumSize(_columnButton->size());

	_grid->addWidget(_columnButton, 1, 2);

	//	type
	QLabel *typelab = new QLabel(
				TableColumn::toString(tableColumn.type()).c_str(), this);
	typelab->setMinimumWidth(3*xWidth);
	typelab->setAlignment(AlignCenter);
	typelab->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	s = "The type of the " + name() + " column";
	QToolTip::add(typelab, s.c_str());
	_grid->addWidget(typelab, 1, 3);

	//	add min/max labels
	_colMinVal = INT(tableColumn.dmin());
	string smin = "min: [" + toString(_colMinVal) + "]";
	QLabel *minLab = new QLabel(this);
	minLab->setText(smin.c_str());
	minLab->setMinimumSize(minLab->sizeHint());
	minLab->setMinimumWidth(9*xWidth);
	minLab->setAlignment(AlignLeft|AlignVCenter);

	_grid->addWidget(minLab, 1, 4);

	_colMaxVal = INT(tableColumn.dmax());
	string smax = "max: [" + toString(_colMaxVal) + "]";
	QLabel *maxLab = new QLabel(this);
	maxLab->setText(smax.c_str());
	maxLab->setMinimumSize(maxLab->sizeHint());
	maxLab->setMinimumWidth(9*xWidth);
	minLab->setAlignment(AlignLeft|AlignVCenter);
	_grid->addWidget(maxLab, 1, 6);

	_minSpinBox = new QSpinBox(_colMinVal, _colMaxVal, 1, this);
	_minSpinBox->setValue(_colMinVal);
	_maxSpinBox = new QSpinBox(_colMinVal, _colMaxVal, 1, this);
	_maxSpinBox->setValue(_colMaxVal);
	s = "Define lower bound of " + name() + " interval";
	QToolTip::add(_minSpinBox, s.c_str());
	s = "Define upper bound of " + name() + " interval";
	QToolTip::add(_maxSpinBox, s.c_str());

	_minSpinBox->setMinimumSize(_minSpinBox->sizeHint());
	_maxSpinBox->setMinimumSize(_maxSpinBox->sizeHint());
	_minSpinBox->setMinimumWidth(8*xWidth);
	_maxSpinBox->setMinimumWidth(8*xWidth);

	_grid->addWidget(_minSpinBox, 1, 5);
	_grid->addWidget(_maxSpinBox, 1, 7);
} // IntegerSelectionGroup::IntegerSelectionGroup()


IntegerSelectionGroup::~IntegerSelectionGroup()
{
	delete _grid;
} // IntegerSelectionGroup::~IntegerSelectionGroup()


void IntegerSelectionGroup::signalLimits()
{
	emit selected(name(), _minSpinBox->value(), _maxSpinBox->value(),
				  _colMinVal, _colMaxVal, _columnButton);
} // signalLimits()


RealSelectionGroup::RealSelectionGroup(QWidget *parent,
		  const TableColumn &tableColumn)
	: SelectionGroup(parent, tableColumn.name()),
	  _minChanged(false), _maxChanged(false)
{
	int xWidth = this->fontMetrics().width('X');

	_grid = new QGridLayout(this, 1, 7, 10);

	//	Real column can be used in image and spectrum/histogram extraction
	_grid->addLayout(setupCheckButtons(true, true), 1, 1);

	_columnButton = new QPushButton(this);
	_columnButton->setText(name().c_str());
	_columnButton->setMinimumSize(_columnButton->sizeHint());
 	_columnButton->setMinimumWidth(12 * xWidth);
	string s = "Append " + name() + " interval filter to selection expression";
	QToolTip::add(_columnButton, s.c_str());

	connect(_columnButton, SIGNAL(pressed()), this, SLOT(signalLimits()));
	this->setMinimumSize(_columnButton->size());
	_grid->addWidget(_columnButton, 1, 2);

	//	type
	QLabel *typelab = new QLabel(
				TableColumn::toString(tableColumn.type()).c_str(), this);
	typelab->setMinimumWidth(3*xWidth);
	typelab->setAlignment(AlignCenter);
	typelab->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	s = "The type of the " + name() + " column";
	QToolTip::add(typelab, s.c_str());
	_grid->addWidget(typelab, 1, 3);

	//	add min/max label
	string smin("min:");
//	if (tableColumn->minSet())
//		smin += " [" + toString(tableColumn->dmin()) + "]";
	QLabel *minLab = new QLabel(this);
	minLab->setText(smin.c_str());
	minLab->setMinimumSize(minLab->sizeHint());
	minLab->setAlignment(AlignLeft|AlignVCenter);
	_grid->addWidget(minLab, 1, 4);

	string smax("max:");
//	if (tableColumn->maxSet())
//		smax += " [" + toString(tableColumn->dmax()) + "]";
	QLabel *maxLab = new QLabel(this);
	maxLab->setText(smax.c_str());
	maxLab->setMinimumSize(maxLab->sizeHint());
	maxLab->setAlignment(AlignLeft|AlignVCenter);
	_grid->addWidget(maxLab, 1, 6);

	_minLineEdit = new QLineEdit(this);
	s = "Define lower bound of " + name() + " interval";
	QToolTip::add(_minLineEdit, s.c_str());
	connect(_minLineEdit, SIGNAL(textChanged(const QString&)),
			this, SLOT(minChanged()));
	_minLineEdit->setMinimumSize(_minLineEdit->sizeHint());
	_grid->addWidget(_minLineEdit, 1, 5);

	_maxLineEdit = new QLineEdit(this);
	s = "Define upper bound of " + name() + " interval";
	QToolTip::add(_maxLineEdit, s.c_str());
	_grid->addWidget(_maxLineEdit, 1, 7);

	connect(_maxLineEdit, SIGNAL(textChanged(const QString&)),
			this, SLOT(maxChanged()));
	_maxLineEdit->setMinimumSize(_maxLineEdit->sizeHint());
} // RealSelectionGroup::RealSelectionGroup()


RealSelectionGroup::~RealSelectionGroup()
{
	delete _grid;
} // RealSelectionGroup::~RealSelectionGroup()


void RealSelectionGroup::signalLimits()
{
	emit selected(name(), atof(_minLineEdit->text()),
				  atof(_maxLineEdit->text()), _minChanged, _maxChanged,
				  _columnButton);
} // RealSelectionGroup::signalLimits()


void RealSelectionGroup::minChanged()
{
	_minChanged = true;
} // RealSelectionGroup::minChanged()


void RealSelectionGroup::maxChanged()
{
	_maxChanged = true;
} // RealSelectionGroup::maxChanged()



StringSelectionGroup::StringSelectionGroup(QWidget *parent,
		  const TableColumn &tableColumn)
	: SelectionGroup(parent, tableColumn.name()),
	  _valueChanged(false)
{
	int xWidth = this->fontMetrics().width('X');

	_grid = new QGridLayout(this, 1, 5, 10);

	//	string columns cannot be used in image, spectrum or rates extraction
	_grid->addLayout(setupCheckButtons(false, false), 1, 1);

	QPushButton *p = new QPushButton(this);
	p->setText(name().c_str());
	p->setMinimumSize(p->sizeHint());
 	p->setMinimumWidth(12 * xWidth);
	string s = "Append " + name() + " equality check to selection expression";
	QToolTip::add(p, s.c_str());
	connect(p, SIGNAL(pressed()), this, SLOT(signalValue()));
	this->setMinimumSize(p->size());
	_grid->addWidget(p, 1, 2);

	//	type
	QLabel *typelab = new QLabel(
				TableColumn::toString(tableColumn.type()).c_str(), this);
	typelab->setMinimumWidth(3*xWidth);
	typelab->setAlignment(AlignCenter);
	typelab->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	s = "The type of the " + name() + " column";
	QToolTip::add(typelab, s.c_str());
	_grid->addWidget(typelab, 1, 3);

	//	add value label
	string value("value:");
	QLabel *vallab = new QLabel(this);
	vallab->setText(value.c_str());
	vallab->setMinimumSize(vallab->sizeHint());
	vallab->setAlignment(AlignLeft|AlignVCenter);
	_grid->addWidget(vallab, 1, 4);

	_valueLineEdit = new QLineEdit(this);
	s = "Define value of " + name() + " for equality check";
	QToolTip::add(_valueLineEdit, s.c_str());
	connect(_valueLineEdit, SIGNAL(textChanged(const QString&)),
			this, SLOT(valueChanged()));
	_valueLineEdit->setMinimumSize(_valueLineEdit->sizeHint());
	_grid->addWidget(_valueLineEdit, 1, 5);
} // StringSelectionGroup::StringSelectionGroup()


StringSelectionGroup::~StringSelectionGroup()
{
	delete _grid;
} // StringSelectionGroup::~StringSelectionGroup()



void StringSelectionGroup::signalValue()
{
	string s(_valueLineEdit->text());
	emit selected(name(), s, _valueChanged);
} // StringSelectionGroup::signalValue()


void StringSelectionGroup::valueChanged()
{
	_valueChanged = true;
} // StringSelectionGroup::valueChanged()



BooleanSelectionGroup::BooleanSelectionGroup(QWidget *parent,
		  const TableColumn &tableColumn)
	: SelectionGroup(parent, tableColumn.name()),
	  _valueChanged(false)
{
	int xWidth = this->fontMetrics().width('X');

	_grid = new QGridLayout(this, 1, 7, 10);

	//	boolean columns cannot be used in image, spectrum or rates extraction
	_grid->addLayout(setupCheckButtons(false, false), 1, 1);

	QPushButton *p = new QPushButton(this);
	p->setText(name().c_str());
	p->setMinimumSize(p->sizeHint());
 	p->setMinimumWidth(12 * xWidth);
	string s = "Append " + name() + " equality check to selection expression";
	QToolTip::add(p, s.c_str());
	connect(p, SIGNAL(pressed()), this, SLOT(signalValue()));
	this->setMinimumSize(p->size());

	_grid->addWidget(p, 1, 2);

	//	type
	QLabel *typelab = new QLabel(
				TableColumn::toString(tableColumn.type()).c_str(), this);
	typelab->setMinimumWidth(3*xWidth);
	typelab->setAlignment(AlignCenter);
	typelab->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	s = "The type of the " + name() + " column";
	QToolTip::add(typelab, s.c_str());
	_grid->addWidget(typelab, 1, 3);

	//	add true/false labels
	string truestr("true");
	QLabel *truelab = new QLabel(this);
	truelab->setText(truestr.c_str());
	truelab->setMinimumSize(truelab->sizeHint());
	_grid->addWidget(truelab, 1, 4);

	string falsestr("false");
	QLabel *falselab = new QLabel(this);
	falselab->setText(falsestr.c_str());
	falselab->setMinimumSize(falselab->sizeHint());
	_grid->addWidget(falselab, 1, 6);

	_valueButton = new QButtonGroup(this);
	_valueButton->setGeometry(0, 0, 0, 0);	//	make the group invisible
	QRadioButton *falseButton = new QRadioButton(this),
				 *trueButton = new QRadioButton(this);
	s = "Define value of " + name() + " to `false' for equality check";
	QToolTip::add(falseButton, s.c_str());
	s = "Define value of " + name() + " to `true' for equality check";
	QToolTip::add(trueButton, s.c_str());

	falseButton->setMinimumSize(falseButton->sizeHint());
	trueButton->setMinimumSize(trueButton->sizeHint());
	_valueButton->insert(falseButton, 0);
	_valueButton->insert(trueButton, 1);

	connect(_valueButton, SIGNAL(pressed(int)), this, SLOT(valueChanged(int)));

	_grid->addWidget(trueButton, 1, 5);
	_grid->addWidget(falseButton, 1, 7);
} // BooleanSelectionGroup::BooleanSelectionGroup()


BooleanSelectionGroup::~BooleanSelectionGroup()
{
	delete _valueButton;
	delete _grid;
} // BooleanSelectionGroup::~BooleanSelectionGroup()



void BooleanSelectionGroup::signalValue()
{
	emit selected(name(), _value, _valueChanged);
} // BooleanSelectionGroup::signalValue()


void BooleanSelectionGroup::valueChanged(int id)
{
	_value = id == 1;
	_valueChanged = true;
} // BooleanSelectionGroup::valueChanged()
