//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                   (c) 1997 - 2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	TableInquiry.cc:	--- implementation of TableColunm/TableInquiry class
//
//	CHANGE HISTORY:
//		UL, 1998-10-22: 1.0.0: - first version
//		UL, 1999-11-03: 1.1.0: - added support for new unsigned types in
//								 dal>=1.80
//		UL, 2000-02-08: 1.2.0: - always define min/max ranges for column
//								 types
//		UL, 2000-07-18: 1.3.0: - determines min/max values of numerical
//								 column values also if now TLMIN/TLMAX
//								 is present
//
//	AUTHOR:
//		U. Lammers (UL), Thu Oct 22 14:24:39 MET DST 1998
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/xmmselect/src/TableInquiry.cc,v 1.2 2002/02/06 16:30:13 ulammers Exp $
//

#include <climits>

#include "TableInquiry.h"
#include "SetTableColumn.h"
#include "DataSubspace.h"
#include "errstr.h"
#include "common.h"

template<class type>
static double min(unsigned N, const type *p, const Column *col)
{
	if (col->hasAttribute("TLMIN"))
		return col->attribute("TLMIN")->asReal();

	type min = p[0];
	for (unsigned n=0; n<N; ++n)
		if (p[n] < min)
			min = p[n];

	return static_cast<double>(min);
} // min()

template<class type>
static double max(unsigned N, const type *p, const Column *col)
{
	if (col->hasAttribute("TLMAX"))
		return col->attribute("TLMAX")->asReal();

	type max = p[0];
	for (unsigned n=0; n<N; ++n)
		if (p[n] > max)
			max = p[n];

	return static_cast<double>(max);
} // max()

//
//	---( TableColumn )--------------------------------------------------------
//
TableColumn::TableColumn(const Column *column) : _dMin(0.), _dMax(0.)
{
	_name = column->name();
	_type = column->dataType();
	unsigned N = column->parent()->rows();
	ColumnData *cd = column->data();
	#define CODE(t1, t2)										\
		case Dal::t1: _dMin = min(N, cd->t2##Data(), column);	\
					  _dMax = max(N, cd->t2##Data(), column);	\
					  break;

	switch (_type) {
		CODE(Real32, real32)
		CODE(Real64, real64)
		CODE(Int8, int8)
		CODE(Int16, int16)
		CODE(Int32, int32)
		CODE(Uint16, uint16)
		CODE(Uint32, uint32)
		default:
			break;
	}
	#undef CODE

	delete cd;
} // TableColumn::TableColumn()


string TableColumn::toString(Dal::DataType dataType)
{
	static char *str[] = { "none", "B", "I8", "U16", "I16", "U32",
						   "I32", "R32", "R64", "S" };

	unsigned i = UNT(dataType);
	if (i >= NUM_OF(str))
		return "unknown";

	return str[i];
} // TableColumn::toString()


//
//	---( TableInquiry )----------------------------------------------------
//
TableInquiry::TableInquiry(const string &tableName) : _table(0)
{
	SetTableColumn st(tableName);
	DataSet *set = st.dataSet();
	Table *table = st.table();
	if (!table)
		//	this should be caught in the Dal
		errstr<<"`"<<tableName<<" is not a table"<<Sas::error("InvalidTable");

	_name = table->qualifiedName();

	//
	//	check if the table has DSS info
	//
	DataSubspace dss(table);
	if (dss.components())
		_dssSelections = dss.selectExpression();

	//
	//	iterator over the columns - get column names and associated types
	//
	for (unsigned n=0; n<table->columns(); ++n)
		push_back(TableColumn(table->column(n)));

	dataSetServer->close(set);
} // TableInquiry::TableInquiry()


Table *TableInquiry::openTable() const
{
	_table = dataSetServer->open(_name, Dal::Read, DataSetServer::Low)->
								table(_name);

	return _table;
} // TableInquiry::openTable()


void TableInquiry::closeTable() const
{
	if (_table)
		dataSetServer->close(_table->parent());
} // TableInquiry::closeTable()


